#!/usr/bin/env python
# -*- coding:utf-8 -*-

import os
import re
import csv
import sys
import datetime
import subprocess

host_list_file = 'host_list.txt'
script_file = '/root/update_log4j_version_for_resetpwdagent_windows.bat'
zip_file = '/root/CloudResetPwdAgent.zip'
exe_file = '/root/7z.exe'
ansible_forks = 10
key_mode, key_file = False, ''


def log_print(log):
    if log:
        date = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        print('%s  %s' % (date, log))


def save_result(csv_file, result):
    headers = ['ip', 'status', 'msg']
    success = result.get('success', [])
    failed = result.get('failed', [])

    with open(csv_file, 'w') as _csv_file:
        csv_writer = csv.writer(_csv_file)
        csv_writer.writerow(headers)
        csv_writer.writerows(success)
        csv_writer.writerows(failed)


def print_result(result):
    success = result.get('success', [])
    failed = result.get('failed', [])

    log_print('Please check the execution result.')
    print('%s%s%s' % ('status'.center(10), 'ip'.center(16), 'msg'.center(30)))
    for ip, _, msg in success:
        print('[SUCCESS] ' + ip.ljust(16) + msg)

    for ip, _, msg in failed:
        print('[FAILED]  ' + ip.ljust(16) + msg)

    print('\nTotal: %d    Success: %d    Failed: %d' % (len(success) + len(failed), len(success), len(failed)))


def parse_block(block, result):
    success_note = 'install ResetPwdAgent success'

    ip_pattern = r'\b(?:25[0-5]\.|2[0-4]\d\.|[01]?\d\d?\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\b'
    ip_list = re.findall(ip_pattern, block)
    if not ip_list:
        safe_exit('Please check the format of %s.' % host_list_file)

    ip = ip_list[0]
    if success_note in block:
        output = (ip, 'success', success_note)
        result['success'].append(output)
    else:
        output = (ip, 'failed', 'Script execute failed')
        result['failed'].append(output)


def parse_log(log_file, result):
    block_content = ''
    block_begin = re.compile(r'^\b(?:25[0-5]\.|2[0-4]\d\.|[01]?\d\d?\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\b'
                              '\s+\|\s+\w+.*$')  # 127.0.0.1 | SUCCESS | rc=0 >>

    with open(log_file, 'r') as exec_log:
        lines = exec_log.readlines()

    for line in lines:
        if block_begin.match(line) and block_content:
            parse_block(block_content, result)
            block_content = ''

        block_content += line
    parse_block(block_content, result)


def parse_exec_log(log_file):
    result_dict = {
        'success': [],
        'failed': []
    }

    parse_log(log_file, result_dict)
    print_result(result_dict)
    save_result('logs/exec_result.csv', result_dict)
    print('You can check the %s for details.' % log_file)


def exec_script():
    log_file = 'logs/exec_origin.log'
    ansible_exec = ['ansible', 'windows',
                    '-i', './hosts',
                    '-m', 'script',
                    '-a', '%s' % script_file,
                    ]

    ansible_exec += ['--private-key', key_file] if key_mode else []

    log_print('Start executing scripts on all hosts, it will take a while..')
    with open(log_file, 'w') as exec_log:
        subprocess.call(ansible_exec, stdout=exec_log)

    parse_exec_log(log_file)


def push_script(log_file, file):
    ansible_copy = ['ansible', 'windows',
                    '-i', './hosts',
                    '-m', 'win_copy',
                    '-a', 'src=%s dest=C:/' % file
                    ]
    if not os.path.exists(file):
        safe_exit('[Error] %s not found.' % file)

    log_print('Start copying the file %s to all hosts.' % file)
    with open(log_file, 'w') as copy_log:
        subprocess.call(ansible_copy, stdout=copy_log)

    with open(log_file, 'r') as copy_log:
        copy_origin = copy_log.read()
        unreachable_pattern = r'\d+\.\d+\.\d+\.\d+\s\|\sUNREACHABLE'
        unreachable_info = re.findall(unreachable_pattern, copy_origin)
        for info in unreachable_info:
            log_print(info)
        if unreachable_info:
            safe_exit('Host(s) is unreachable, please check the network or password of user root.'
                      '\nYou can check the %s for details.' % log_file)


def build_ansible_cfg():
    ansible_cfg = '/etc/ansible/ansible.cfg'

    if not os.path.exists(host_list_file):
        safe_exit('[Error] %s not found.' % host_list_file)

    with open(host_list_file, 'r') as host_list:
        origin_host = host_list.readlines()
        with open('hosts', 'w') as hosts:
            hosts.write('[windows]\n')
            for line in origin_host:
                item = re.sub(r',\s*', ' ansible_ssh_port=5985 ansible_connection="winrm" ansible_winrm_server_cert_validation=ignore ansible_winrm_transport="ntlm" ansible_ssh_user="Administrator" ansible_ssh_pass=', line)
                hosts.write(item)

    if not os.path.exists(ansible_cfg):
        safe_exit('Please confirm that the ansible has been installed.')


def rename_exe_file():
    file = ''
    file_list = os.listdir("/root")
    for tmpfile in file_list:
        if '7z' in tmpfile:
            file = tmpfile
    os.rename(file, exe_file)


def prepare_env():
    build_ansible_cfg()
    rename_exe_file()
    if key_mode and not os.path.exists(key_file):
        safe_exit('[Error] key-file %s not found.' % key_file)

    if not os.path.exists(script_file):
        safe_exit('[Error] %s not found.' % script_file)

    if not os.path.exists('./logs/'):
        os.makedirs('./logs/')


def clean_env():
    if os.path.exists('hosts'):
        os.remove('hosts')


def safe_exit(note=''):
    clean_env()
    log_print(note)
    exit()


if __name__ == '__main__':
    if len(sys.argv) >= 2:
        key_mode, key_file = True, sys.argv[1]
    prepare_env()
    push_script('logs/copy_CloudResetPwdAgent.log', zip_file)
    push_script('logs/copy_origin.log', script_file)
    push_script('logs/copy_exe.log', exe_file)
    exec_script()
    safe_exit()
